using System.Text.Json;
using Microsoft.AspNetCore.SignalR;
using Todo.Api.Hubs.Models;
using Todo.Core.Interfaces.Persistence;

namespace Todo.Api.Hubs
{
    public class TodoHub : Hub
    {
        private readonly ITodoRepository _todoRepository;

        public TodoHub(ITodoRepository todoRepository)
        {
            _todoRepository = todoRepository;
        }

        public async Task CreateTodo(string todoTitle, string projectName)
        {
            if (todoTitle is null)
                throw new ArgumentException("title cannot be null");
            var _ = await _todoRepository.CreateTodoAsync(todoTitle, projectName);

            var todos = await _todoRepository.GetNotDoneTodos();
            var serializedTodos =
                JsonSerializer.Serialize(todos
                    .Select(t => new TodoResponse { Id = t.Id, Title = t.Title, Project = t.Project })
                    .ToList());

            await Clients.Caller.SendAsync("getInboxTodos", serializedTodos);
        }

        public async Task UpdateTodo(string todoId, bool todoStatus)
        {
            await _todoRepository.UpdateTodoStatus(todoId, todoStatus);

            var todos = await _todoRepository.GetNotDoneTodos();
            var serializedTodos =
                JsonSerializer.Serialize(todos
                    .Select(t => new TodoResponse
                        { Id = t.Id, Title = t.Title, Status = t.Status, Project = t.Project })
                    .ToList());

            await Clients.Caller.SendAsync("getInboxTodos", serializedTodos);
        }

        public async Task GetTodos()
        {
            var todos = await _todoRepository.GetTodosAsync();
            var serializedTodos = JsonSerializer.Serialize(todos
                .Select(t => new TodoResponse { Id = t.Id, Title = t.Title, Status = t.Status, Project = t.Project })
                .ToList());

            await Clients.Caller.SendAsync("todos", serializedTodos);
        }

        public async Task GetInboxTodos()
        {
            var todos = await _todoRepository.GetNotDoneTodos();
            var serializedTodos = JsonSerializer.Serialize(todos
                .Select(t => new TodoResponse { Id = t.Id, Title = t.Title, Status = t.Status, Project = t.Project })
                .ToList());

            await Clients.Caller.SendAsync("getInboxTodos", serializedTodos);
        }

        public async Task GetTodo(string todoId)
        {
            var todo = await _todoRepository.GetTodoByIdAsync(todoId);
            var serializedTodo = JsonSerializer.Serialize(new TodoResponse()
            {
                Id = todo.Id,
                Project = todo.Project,
                Status = todo.Status,
                Title = todo.Title,
            });

            await Clients.Caller.SendAsync("getTodo", serializedTodo);
        }

        public async Task ReplaceTodo(string updateTodoRequest)
        {
            var updateTodo = JsonSerializer.Deserialize<UpdateTodoRequest>(updateTodoRequest);
            if (updateTodo is null)
                throw new InvalidOperationException("Could not parse invalid updateTodo");

            var updatedTodo = await _todoRepository.UpdateTodoAsync(new Core.Entities.Todo()
            {
                Id = updateTodo.Id,
                Project = updateTodo.Project,
                Status = updateTodo.Status,
                Title = updateTodo.Title
            });

            var serializedTodo = JsonSerializer.Serialize(new TodoResponse()
            {
                Id = updatedTodo.Id,
                Project = updatedTodo.Project,
                Status = updatedTodo.Status,
                Title = updatedTodo.Title,
            });

            await Clients.Caller.SendAsync("getTodo", serializedTodo);
        }
    }
}